/*******************************************************************************
 * CS559 Spring 2008 - Project 2 - Trains and Roller Coasters
 *
 * Example Solution, by Chi Man Liu
 * begin 3/13/2008
 *
 * File: P2GlWindow.h
 * The OpenGL window that contains most of the logic and drawing routines.
 * Important methods:
 *   draw: draw stuff on the screen. Called whenever Fltk thinks the screen
 *         needd to be updated (e.g. window resized, covered/uncovered by
 *         other programs), or when we invoke damage(1).
 *   handle: handle events such as keypresses and mouse movements.
 *   advanceAnimation: update the world by one frame. In this sample solution,
 *                     only the train moves.
 ******************************************************************************/
#ifndef P2GLWINDOW_H
#define P2GLWINDOW_H

// the FlTk headers have lots of warnings - these are bad, but there's not
// much we can do about them. We enclose the #include's by #pragma's to
// suppress warnings generated by the Fltk library.
#pragma warning(push)
#pragma warning(disable:4311)		// convert void* to long
#pragma warning(disable:4312)		// convert long to void*
#include <Fl/Fl_Gl_Window.h>
#pragma warning(pop)

#include "Track.h"

class P2GlWindow : public Fl_Gl_Window {
public:
	// constructor and destructor
	P2GlWindow(int x, int y, int width, int height, char* title);
	virtual ~P2GlWindow();

	// initialize GL operations
	void initGL();
	// overwrite the draw function in Fl_Gl_Window
	virtual void draw();

	// move one frame forward, and draw
	void advanceAnimation();

	// toggle between train view and world view
	void toggleView();

private:
	// override event handler of window to deal with keypresses and mouse
	int handle(int event);

	//////////////////// DRAWING ROUTINES ////////////////////

	// draw the ground plane
	void drawGroundPlane();
	// draw the train
	void drawTrain();

	//////////////////////// ANIMATION CONTROL ///////////////////////

	// Time passed since the starting point.
	// This is not the actual real-world time - you can think of it
	// as the number of timesteps the train has moved from the beginning.
	// Based on this we calculate the train's position and direction.
	// In the sample solution, 1 loop = 1 unit of time, so timeElapsed
	// is always a fraction between 0 and 1.
	double timeElapsed;
	// The amount to increment timeElapsed after each frame of animation.
	// The larger its value, the faster the animation.
	// THIS IS NOT THE FRAME PER SECOND. You should differentiate
	// between real world time and the "time" we defined here (as the
	// fraction of the track the train has completed).
	double timePerFrame;

	/////////////////////// SIMPLE CAMERA CONTROL //////////////////////

	// amount of camera rotation (in degree)
	double cameraRotation;
	// degree of camera above horizon
	double cameraPitch;

	////////////////////// OTHER PRIVATE STUFF /////////////////////

	// the track
	Track *track;
	// true - train view; false - world view
	bool trainViewOn;

};

#endif

