/*******************************************************************************
 * CS559 Spring 2008 - Project 2 - Trains and Roller Coasters
 *
 * Example Solution, by Chi Man Liu
 * begin 3/13/2008
 *
 * This program draws a ground plane (checkerboard), a simple track (circle)
 * and a train (cube) in an OpenGL window along with two buttons - one is for
 * playing/pausing the animation, the other is for toggling between world view
 * and train view. The camera can be rotated using the arrow keys.
 *
 * File: main.cpp
 * creates the main user interface (main window, OpenGL window and a few
 * buttons. also contains callback functions for buttons and an idle callback
 * function which is used for animation. The meat of this program is in the
 * P2GlWindow class.
 ******************************************************************************/

// the FlTk headers have lots of warnings - these are bad, but there's not
// much we can do about them. We enclose the #include's by #pragma's to
// suppress warnings generated by the Fltk library.
#pragma warning(push)
#pragma warning(disable:4311)		// convert void* to long
#pragma warning(disable:4312)		// convert long to void*
#include <Fl/Fl.h>
#include <Fl/Fl_Double_Window.h>
#include <Fl/Fl_Light_Button.h>
#include <Fl/Fl_Button.h>
#pragma warning(pop)

// for controlling the frame rate
#include <ctime>

// our OpenGL window
#include "P2GlWindow.h"


// The OpenGL window. We make this global because the idle callback
// function needs it.
P2GlWindow *p2GlWindow= NULL;

// Idle callback function - animate, keeping frame rate not greater
// than 30 fps. This function is called repeatedly when the window
// is not handling any event, provided that it has already been
// installed (see playButtonCallback below).
void idleCallback(void *data) {
	static int lastTime = clock();
	// this keeps it from moving too fast
	if (clock() - lastTime > CLOCKS_PER_SEC / 30) {
		lastTime = clock();
		// advance animation by one frame
		p2GlWindow->advanceAnimation();
	}
}

// Callback function - play/pause animation
void playButtonCallback(Fl_Widget *w) {
	// get the current state ("on" or "off") of the button
	bool play = ((Fl_Light_Button*)w)->value() == 1;
	if (play) {
        // install idle callback. This will start the animation.
		Fl::add_idle(idleCallback, NULL);
	} else {
        // uninstall idle callback. This will stop the animation.
		Fl::remove_idle(idleCallback, NULL);
	}
}

// Callback function - toggle view
void cameraButtonCallback(Fl_Widget *w) {
	p2GlWindow->toggleView();
}

// the main function. mainly UI stuff.
int main(int argc, char** args) {

	// create a doubly-buffered main window that contains our OpenGL window
	// and buttons
	Fl_Double_Window *mainWindow = new Fl_Double_Window(640, 510, "CS559 Project 2 Sample Solution");

	// start adding stuff to the window.
	// you can easily add buttons or other UI components to a window by new-ing
	// them between mainWindow->begin() and mainWindow->end().
	mainWindow->begin();

	// create the OpenGL window
	p2GlWindow = new P2GlWindow(0, 0, 640, 480, "CS559 Project 2 Sample Solution");
	
	// add a play button
	Fl_Light_Button *playButton = new Fl_Light_Button(0, 485, 120, 25, "Play");
	// initialize button to "off" state
	playButton->value(0);
	// add callback function for play button
	playButton->callback(playButtonCallback);
	
	// add a "toggle camera" button
	Fl_Button *cameraButton = new Fl_Button(130, 485, 150, 25, "Toggle View");
	// add callback
	cameraButton->callback(cameraButtonCallback);

	// stop adding stuff
	mainWindow->end();

	// show the window
	mainWindow->show();

	// run it
	Fl::run();

	// free up memory
	delete p2GlWindow;
	delete mainWindow;

	return 0;
}
